@@ -254,6 +254,8 @@ class Form:
         self.type.form = self
 
         self.props = {}     # name: Prop()
+        self.ifaces = {}    # name: <ifacedef>
+
         self.refsout = None
 
         self.locked = False
@@ -419,6 +421,7 @@ class Model:
         self.types = {} # name: Type()
         self.forms = {} # name: Form()
         self.props = {} # (form,name): Prop() and full: Prop()
+        self.ifaces = {}  # name: <ifdef>
         self.tagprops = {} # name: TagProp()
         self.formabbr = {} # name: [Form(), ... ]
         self.modeldefs = []
@@ -427,6 +430,8 @@ class Model:
 
         self.propsbytype = collections.defaultdict(list) # name: Prop()
         self.arraysbytype = collections.defaultdict(list)
+        # TODO use this for <nodes> -> foo:iface
+        self.formsbyiface = collections.defaultdict(list)
 
         self._type_pends = collections.defaultdict(list)
         self._modeldef = {
@@ -513,7 +518,8 @@ class Model:
         item = s_types.NodeProp(self, 'nodeprop', info, {})
         self.addBaseType(item)
 
-        info = {'doc': 'A potentially huge/tiny number. [x] <= 170141183460469231731687 with a fractional precision of 15 decimal digits.'}
+        info = {'doc': 'A potentially huge/tiny number. [x] <= 170141183460469231731687 with a fractional '
+                       'precision of 15 decimal digits.'}
         item = s_types.HugeNum(self, 'hugenum', info, {})
         self.addBaseType(item)
 
@@ -553,6 +559,7 @@ class Model:
         mdef['forms'] = [f.getFormDef() for f in self.forms.values()]
         mdef['univs'] = [u.getPropDef() for u in self.univs.values()]
         mdef['tagprops'] = [t.getTagPropDef() for t in self.tagprops.values()]
+        mdef['interfaces'] = list(self.ifaces.items())
         return [('all', mdef)]
 
     def getModelDict(self):
@@ -560,6 +567,7 @@ class Model:
             'types': {},
             'forms': {},
             'tagprops': {},
+            'interfaces': self.ifaces.copy()
         }
 
         for tobj in self.types.values():
@@ -599,6 +607,13 @@ class Model:
                 "tagprops":(
                     (tagpropname, (typename, typeopts), {info}),
                 )
+                "interfaces":(
+                    (ifacename, {
+                        'props': ((propname, (typename, typeopts), {info}),),
+                        'doc': docstr,
+                        'interfaces': (ifacename,)
+                    }),
+                )
             }
 
         Args:
@@ -626,6 +641,11 @@ class Model:
                 typeinfo['custom'] = custom
                 self.addType(typename, basename, typeopts, typeinfo)
 
+        # load all the interfaces...
+        for _, mdef in mods:
+            for name, info in mdef.get('interfaces', ()):
+                self.addIface(name, info)
+
         # Load all the universal properties
         for _, mdef in mods:
             for univname, typedef, univinfo in mdef.get('univs', ()):
@@ -687,6 +707,11 @@ class Model:
             propname, typedef, propinfo = propdef
             self._addFormProp(form, propname, typedef, propinfo)
 
+        # interfaces are listed in typeinfo for the form to
+        # maintain backward compatibility for populated models
+        for ifname in form.type.info.get('interfaces', ()):
+            self._addFormIface(form, ifname)
+
         return form
 
     def delForm(self, formname):
@@ -698,9 +723,16 @@ class Model:
         if isinstance(form.type, s_types.Array):
             self.arraysbytype[form.type.arraytype.name].remove(form)
 
+        for ifname in form.ifaces.keys():
+            self.formsbyiface[ifname].remove(form)
+
         self.forms.pop(formname, None)
         self.props.pop(formname, None)
 
+    def addIface(self, name, info):
+        # TODO should we add some meta-props here for queries?
+        self.ifaces[name] = info
+
     def delType(self, typename):
 
         _type = self.types.get(typename)
@@ -755,6 +787,24 @@ class Model:
         self.props[prop.full] = prop
         return prop
 
+    def _addFormIface(self, form, name):
+
+        iface = self.ifaces.get(name)
+
+        if iface is None:
+            mesg = f'Form {form.name} depends on non-existant interface: {name}'
+            raise s_exc.NoSuchName(mesg=mesg)
+
+        for propname, typedef, propinfo in iface.get('props', ()):
+            self._addFormProp(form, propname, typedef, propinfo)
+
+        # TODO use this to allow storm: +foo:iface
+        form.ifaces[name] = iface
+        self.formsbyiface[name].append(form)
+
+        for ifname in iface.get('interfaces', ()):
+            self._addFormIface(form, ifname)
+
     def delTagProp(self, name):
         return self.tagprops.pop(name)
 
