@@ -5,6 +5,7 @@ import shutil
 import socket
 import asyncio
 import logging
+import tarfile
 import argparse
 import datetime
 import functools
@@ -23,8 +24,10 @@ import synapse.lib.base as s_base
 import synapse.lib.boss as s_boss
 import synapse.lib.coro as s_coro
 import synapse.lib.hive as s_hive
+import synapse.lib.link as s_link
 import synapse.lib.const as s_const
 import synapse.lib.nexus as s_nexus
+import synapse.lib.scope as s_scope
 import synapse.lib.config as s_config
 import synapse.lib.health as s_health
 import synapse.lib.output as s_output
@@ -73,6 +76,58 @@ def adminapi(log=False):
 
     return decrfunc
 
+async def _doIterBackup(path, chunksize=1024):
+    '''
+    Create tarball and stream bytes.
+
+    Args:
+        path (str): Path to the backup.
+
+    Yields:
+        (bytes): File bytes
+    '''
+    output_filename = path + '.tar.gz'
+    link0, file1 = await s_link.linkfile()
+
+    def dowrite(fd):
+        with tarfile.open(output_filename, 'w|gz', fileobj=fd) as tar:
+            tar.add(path, arcname=os.path.basename(path))
+        fd.close()
+
+    coro = s_coro.executor(dowrite, file1)
+
+    while True:
+        byts = await link0.recv(chunksize)
+        if not byts:
+            return
+        yield byts
+
+async def _iterBackupWork(path, linkinfo, done):
+    '''
+    Inner setup for backup streaming.
+
+    Args:
+        path (str): Path to the backup.
+        linkinfo(dict): Link info dictionary.
+        done (multiprocessing.Queue): TX Queue.
+
+    Returns:
+        None: Returns None.
+    '''
+    link = await s_link.fromspawn(linkinfo)
+    await s_daemon.t2call(link, _doIterBackup, (path,), {})
+
+    wasfini = link.isfini
+    await link.fini()
+
+    await s_coro.executor(done.put, wasfini)
+
+def _iterBackupProc(path, linkinfo, done):
+    '''
+    Multiprocessing target for streaming a backup.
+    '''
+    asyncio.run(_iterBackupWork(path, linkinfo, done))
+
 class CellApi(s_base.Base):
 
     async def __anit__(self, cell, link, user):
@@ -191,8 +246,8 @@ class CellApi(s_base.Base):
         return await self.cell.kill(self.user, iden)
 
     @adminapi(log=True)
-    async def addUser(self, name, passwd=None, email=None):
-        return await self.cell.addUser(name, passwd=passwd, email=email)
+    async def addUser(self, name, passwd=None, email=None, iden=None):
+        return await self.cell.addUser(name, passwd=passwd, email=email, iden=iden)
 
     @adminapi(log=True)
     async def delUser(self, iden):
@@ -494,6 +549,39 @@ class CellApi(s_base.Base):
         '''
         return await self.cell.delBackup(name)
 
+    @adminapi()
+    async def iterBackupArchive(self, name):
+        '''
+        Retrieve a backup by name as a compressed stream of bytes.
+
+        Note: Compression and streaming will occur from a separate process.
+
+        Args:
+            name (str): The name of the backup to retrieve.
+        '''
+        await self.cell.iterBackupArchive(name, user=self.user)
+
+        # Make this a generator
+        if False: # pragma: no cover
+            yield
+
+    @adminapi()
+    async def iterNewBackupArchive(self, name=None, remove=False):
+        '''
+        Run a new backup and return it as a compressed stream of bytes.
+
+        Note: Compression and streaming will occur from a separate process.
+
+        Args:
+            name (str): The name of the backup to retrieve.
+            remove (bool): Delete the backup after streaming.
+        '''
+        await self.cell.iterNewBackupArchive(user=self.user, name=name, remove=remove)
+
+        # Make this a generator
+        if False: # pragma: no cover
+            yield
+
     @adminapi()
     async def getDiagInfo(self):
         return {
@@ -585,6 +673,22 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
             'description': 'An AHA client certificate CN to register as a local admin user.',
             'type': 'string',
         },
+        'aha:svcinfo': {
+            'description': 'An AHA svcinfo object. If set, this overrides self discovered Aha service information.',
+            'type': 'object',
+            'properties': {
+                'urlinfo': {
+                    'type': 'object',
+                    'properties': {
+                        'host': {'type': 'string'},
+                        'port': {'type': 'integer'},
+                        'schema': {'type': 'string'}
+                    },
+                    'required': ('host', 'port', 'scheme', )
+                }
+            },
+            'required': ('urlinfo', ),
+        }
     }
 
     BACKUP_SPAWN_TIMEOUT = 4.0
@@ -752,7 +856,12 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
         ahaurl = self.conf.get('aha:registry')
         if ahaurl is not None:
 
-            self.ahaclient = await s_telepath.addAhaUrl(ahaurl)
+            info = await s_telepath.addAhaUrl(ahaurl)
+            self.ahaclient = info.get('client')
+            if self.ahaclient is None:
+                self.ahaclient = await s_telepath.Client.anit(info.get('url'))
+                self.ahaclient._fini_atexit = True
+                info['client'] = self.ahaclient
 
             async def finiaha():
                 await s_telepath.delAhaUrl(ahaurl)
@@ -1056,6 +1165,7 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
             task.add_done_callback(functools.partial(done, self))
 
             if wait:
+                logger.info(f'Waiting for backup to complete [{name}]')
                 await task
 
             return name
@@ -1070,6 +1180,8 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
         '''
         A task that backs up the cell to the target directory
         '''
+        logger.info(f'Starting backup to [{dirn}]')
+
         await self.boss.promote('backup', self.auth.rootuser)
         slabs = s_lmdbslab.Slab.getSlabsInDir(self.dirn)
         assert slabs
@@ -1113,12 +1225,17 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
                 if proc.exitcode:
                     raise s_exc.SpawnExit(code=proc.exitcode)
 
-            return await s_coro.executor(waitforproc)
+            retn = await s_coro.executor(waitforproc)
 
         except (asyncio.CancelledError, Exception):
+            logger.exception('Error performing backup to [{dirn}]')
             proc.terminate()
             raise
 
+        else:
+            logger.info(f'Backup completed to [{dirn}]')
+            return retn
+
     @staticmethod
     def _backupProc(pipe, srcdir, dstdir, lmdbpaths):
         '''
@@ -1147,8 +1264,9 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
         if not os.path.isfile(cellguid):
             mesg = 'Specified backup path has no cell.guid file.'
             raise s_exc.BadArg(mesg=mesg)
-
+        logger.info(f'Removing backup for [{path}]')
         await s_coro.executor(shutil.rmtree, path, ignore_errors=True)
+        logger.info(f'Backup removed from [{path}]')
 
     async def getBackups(self):
         self._reqBackConf()
@@ -1171,6 +1289,108 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
         walkpath(self.backdirn)
         return backups
 
+    async def iterBackupArchive(self, name, user):
+
+        path = self._reqBackDirn(name)
+        cellguid = os.path.join(path, 'cell.guid')
+        if not os.path.isfile(cellguid):
+            mesg = 'Specified backup path has no cell.guid file.'
+            raise s_exc.BadArg(mesg=mesg)
+
+        link = s_scope.get('link')
+        linkinfo = link.getSpawnInfo()
+
+        await self.boss.promote('backup:stream', user=user)
+
+        ctx = multiprocessing.get_context('spawn')
+        done = ctx.Queue()
+
+        proc = None
+        mesg = 'Streaming complete'
+
+        def getproc():
+            proc = ctx.Process(target=_iterBackupProc, args=(path, linkinfo, done))
+            proc.start()
+            return proc
+
+        try:
+            proc = await s_coro.executor(getproc)
+
+            def waitproc():
+                proc.join()
+                return done.get()
+
+            if await s_coro.executor(waitproc):
+                await link.fini()
+
+        except (asyncio.CancelledError, Exception) as e:
+
+            if not isinstance(e, asyncio.CancelledError):
+                logger.exception('Error during backup streaming.')
+
+            if proc:
+                await s_coro.executor(done.put, False)
+                proc.terminate()
+
+            mesg = repr(e)
+            raise
+
+        finally:
+            raise s_exc.DmonSpawn(mesg=mesg)
+
+    async def iterNewBackupArchive(self, user, name=None, remove=False):
+
+        if name is None:
+            name = time.strftime('%Y%m%d%H%M%S', datetime.datetime.now().timetuple())
+
+        path = self._reqBackDirn(name)
+        if os.path.isdir(path):
+            mesg = 'Backup with name already exists'
+            raise s_exc.BadArg(mesg=mesg)
+
+        proc = None
+        mesg = 'Streaming complete'
+
+        try:
+            await self.runBackup(name)
+            linkinfo = s_scope.get('link').getSpawnInfo()
+
+            await self.boss.promote('backup:stream', user=user)
+
+            ctx = multiprocessing.get_context('spawn')
+            done = ctx.Queue()
+
+            def getproc():
+                proc = ctx.Process(target=_iterBackupProc, args=(path, linkinfo, done))
+                proc.start()
+                return proc
+
+            proc = await s_coro.executor(getproc)
+
+            def waitproc():
+                proc.join()
+                return done.get()
+
+            if await s_coro.executor(waitproc):
+                await link.fini()
+
+        except (asyncio.CancelledError, Exception) as e:
+
+            if not isinstance(e, asyncio.CancelledError):
+                logger.exception('Error during backup streaming.')
+
+            if proc:
+                await s_coro.executor(done.put, False)
+                proc.terminate()
+
+            mesg = repr(e)
+            raise
+
+        finally:
+            if remove:
+                await s_coro.executor(shutil.rmtree, path, ignore_errors=True)
+            raise s_exc.DmonSpawn(mesg=mesg)
+
     async def isUserAllowed(self, iden, perm, gateiden=None):
         user = self.auth.user(iden)
         if user is None:
@@ -1241,8 +1461,8 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
 
         await self.fire('user:mod', act='revoke', user=useriden, role=roleiden)
 
-    async def addUser(self, name, passwd=None, email=None):
-        user = await self.auth.addUser(name, passwd=passwd, email=email)
+    async def addUser(self, name, passwd=None, email=None, iden=None):
+        user = await self.auth.addUser(name, passwd=passwd, email=email, iden=iden)
         await self.fire('user:mod', act='adduser', name=name)
         return user.pack(packroles=True)
 
@@ -1690,11 +1910,11 @@ class Cell(s_nexus.Pusher, s_telepath.Aware):
 
         opts = pars.parse_args(argv)
 
+        s_common.setlogging(logger, defval=opts.log_level)
+
         conf.setConfFromOpts(opts)
         conf.setConfFromEnvs()
 
-        s_common.setlogging(logger, defval=opts.log_level)
-
         cell = await cls.anit(opts.dirn, conf=conf)
 
         try:
