@@ -68,6 +68,12 @@ class HandlerBase:
             self.add_header('Access-Control-Allow-Credentials', 'true')
             self.add_header('Access-Control-Allow-Headers', 'Content-Type')
 
+    def getAuthCell(self):
+        '''
+        Return a reference to the cell used for auth operations.
+        '''
+        return self.cell
+
     def options(self):
         self.set_status(204)
         self.finish()
@@ -85,8 +91,8 @@ class HandlerBase:
     def check_origin(self, origin):
         return self.isOrigHost(origin)
 
-    def getJsonBody(self):
-        return self.loadJsonMesg(self.request.body)
+    def getJsonBody(self, validator=None):
+        return self.loadJsonMesg(self.request.body, validator=validator)
 
     def sendRestErr(self, code, mesg):
         self.set_header('Content-Type', 'application/json')
@@ -100,14 +106,22 @@ class HandlerBase:
         self.set_header('Content-Type', 'application/json')
         return self.write({'status': 'ok', 'result': valu})
 
-    def loadJsonMesg(self, byts):
+    def loadJsonMesg(self, byts, validator=None):
         try:
-            return json.loads(byts)
+            item = json.loads(byts)
+            if validator is not None:
+                validator(item)
+            return item
+
+        except s_exc.SchemaViolation as e:
+            self.sendRestErr('SchemaViolation', str(e))
+            return None
+
         except Exception:
             self.sendRestErr('SchemaViolation', 'Invalid JSON content.')
             return None
 
-    def sendAuthReqired(self):
+    def sendAuthRequired(self):
         self.set_header('WWW-Authenticate', 'Basic realm=synapse')
         self.set_status(401)
         self.sendRestErr('NotAuthenticated', 'The session is not logged in.')
@@ -115,14 +129,14 @@ class HandlerBase:
     async def reqAuthUser(self):
         if await self.authenticated():
             return True
-        self.sendAuthReqired()
+        self.sendAuthRequired()
         return False
 
     async def reqAuthAdmin(self):
 
         user = await self.user()
         if user is None:
-            self.sendAuthReqired()
+            self.sendAuthRequired()
             return False
 
         if not user.isAdmin():
@@ -131,48 +145,6 @@ class HandlerBase:
 
         return True
 
-    async def reqAuthAllowed(self, path):
-        '''
-        Helper method that subclasses can use for user permission checking.
-
-        Args:
-            path: Tuple of permission path components to check.
-
-        Notes:
-            This will call reqAuthUser() to ensure that there is a valid user.
-            If this returns False, the handler should return since the the
-            status code and resulting error message will already have been
-            sent to the requester.
-
-        Examples:
-
-            Define a handler which checks for ``syn:test`` permission::
-
-                class ReqAuthHandler(s_httpapi.Handler):
-                    async def get(self):
-                        if not await self.reqAuthAllowed(('syn:test', )):
-                            return
-                     return self.sendRestRetn({'data': 'everything is awesome!'})
-
-        Returns:
-            bool: True if the user is allowed; False if the user is not allowed.
-
-        Raises:
-            s_exc.AuthDeny: If the permission is not allowed.
-
-        '''
-
-        if not await self.reqAuthUser():
-            return False
-
-        user = await self.user()
-        if not user.allowed(path):
-            self.set_status(403)
-            mesg = f'User {user.iden} ({user.name}) must have permission {".".join(path)}'
-            self.sendRestErr('AuthDeny', mesg)
-            return False
-        return True
-
     async def sess(self, gen=True):
 
         if self._web_sess is None:
@@ -229,13 +201,49 @@ class HandlerBase:
         self._web_user = user
         return user
 
-    async def username(self):
+    async def useriden(self):
+        '''
+        Return the user iden of the current session user.
+
+        NOTE: APIs should migrate toward using this rather than the heavy
+              Handler.user() API to facilitate reuse of handler objects with
+              telepath references.
+        '''
         user = await self.user()
-        if user is not None:
-            return user.name
+        if user is None:
+            return None
+        return user.iden
+
+    async def allowed(self, perm, gateiden=None):
+        '''
+        Return true if there is a logged in user with the given permissions.
+
+        NOTE: This API sets up HTTP response values if it returns False.
+
+        NOTE: This API uses the Handler.getAuthCell() abstraction and is safe for use
+              in split-auth cells.
+        '''
+        authcell = self.getAuthCell()
+
+        useriden = await self.useriden()
+        if useriden is None:
+            self.sendAuthRequired()
+            return False
+
+        if await authcell.isUserAllowed(useriden, perm, gateiden=gateiden):
+            return True
+
+        udef = await authcell.getUserDef(useriden)
+
+        username = udef.get('name')
+
+        self.set_status(403)
+        mesg = f'User ({username}) must have permission {".".join(perm)}'
+        self.sendRestErr('AuthDeny', mesg)
+        return False
 
     async def authenticated(self):
-        return await self.user() is not None
+        return await self.useriden() is not None
 
     async def getUserBody(self):
         '''
@@ -285,11 +293,13 @@ class Handler(HandlerBase, t_web.RequestHandler):
         if opts is None:
             opts = {}
 
-        user = await self.user()
+        authcell = self.getAuthCell()
+        useriden = await self.useriden()
 
-        opts.setdefault('user', user.iden)
-        if opts.get('user') != user.iden:
-            user.confirm(('impersonate',))
+        opts.setdefault('user', useriden)
+        if opts.get('user') != useriden:
+            if not await self.allowed(('impersonate',)):
+                return None
 
         return opts
 
@@ -309,7 +319,14 @@ class StreamHandler(Handler):
         raise s_exc.NoSuchImpl(mesg='data_received must be implemented by subclasses.',
                                name='data_received')
 
-class StormNodesV1(Handler):
+class StormHandler(Handler):
+
+    def getCore(self):
+        # add an abstraction to allow subclasses to dictate how
+        # a reference to the cortex is returned from the handler.
+        return self.cell
+
+class StormNodesV1(StormHandler):
 
     async def post(self):
         return await self.get()
@@ -327,56 +344,65 @@ class StormNodesV1(Handler):
         await self.cell.boss.promote('storm', user=user, info={'query': query})
 
         opts = await self._reqValidOpts(opts)
+        if opts is None:
+            return
 
         view = self.cell._viewFromOpts(opts)
         async for pode in view.iterStormPodes(query, opts=opts):
             self.write(json.dumps(pode))
             await self.flush()
 
-class StormV1(Handler):
+class StormV1(StormHandler):
 
     async def post(self):
         return await self.get()
 
     async def get(self):
 
-        user, body = await self.getUserBody()
-        if body is s_common.novalu:
+        if not await self.reqAuthUser():
+            return
+
+        body = self.getJsonBody()
+        if body is None:
             return
 
-        # dont allow a user to be specified
         opts = body.get('opts')
         query = body.get('query')
 
         # Maintain backwards compatibility with 0.1.x output
         opts = await self._reqValidOpts(opts)
+        if opts is None:
+            return
+
         opts['editformat'] = 'splices'
 
-        await self.cell.boss.promote('storm', user=user, info={'query': query})
-
-        async for mesg in self.cell.storm(query, opts=opts):
+        async for mesg in self.getCore().storm(query, opts=opts):
             self.write(json.dumps(mesg))
             await self.flush()
 
-class StormCallV1(Handler):
+class StormCallV1(StormHandler):
 
     async def post(self):
         return await self.get()
 
     async def get(self):
 
-        user, body = await self.getUserBody()
-        if body is s_common.novalu:
+        if not await self.reqAuthUser():
+            return
+
+        body = self.getJsonBody()
+        if body is None:
             return
 
-        # dont allow a user to be specified
         opts = body.get('opts')
         query = body.get('query')
 
         opts = await self._reqValidOpts(opts)
+        if opts is None:
+            return
 
         try:
-            ret = await self.cell.callStorm(query, opts=opts)
+            ret = await self.getCore().callStorm(query, opts=opts)
         except s_exc.SynErr as e:
             mesg = e.get('mesg', str(e))
             return self.sendRestErr(e.__class__.__name__, mesg)
@@ -388,33 +414,37 @@ class StormCallV1(Handler):
         else:
             return self.sendRestRetn(ret)
 
-class StormExportV1(Handler):
+class StormExportV1(StormHandler):
 
     async def post(self):
         return await self.get()
 
     async def get(self):
 
-        user, body = await self.getUserBody()
-        if body is s_common.novalu: # pragma: no cover
+        if not await self.reqAuthUser():
+            return
+
+        body = self.getJsonBody()
+        if body is None:
             return
 
-        # dont allow a user to be specified
         opts = body.get('opts')
         query = body.get('query')
 
-        # Maintain backwards compatibility with 0.1.x output
         opts = await self._reqValidOpts(opts)
+        if opts is None:
+            return
 
         try:
             self.set_header('Content-Type', 'application/x-synapse-nodes')
-            async for pode in self.cell.exportStorm(query, opts=opts):
+            async for pode in self.getCore().exportStorm(query, opts=opts):
                 self.write(s_msgpack.en(pode))
+                await self.flush()
 
         except Exception as e:
             return self.sendRestExc(e)
 
-class ReqValidStormV1(Handler):
+class ReqValidStormV1(StormHandler):
 
     async def post(self):
         return await self.get()
@@ -848,7 +878,7 @@ class ModelV1(Handler):
 class HealthCheckV1(Handler):
 
     async def get(self):
-        if not await self.reqAuthAllowed(('health', )):
+        if not await self.allowed(('health', )):
             return
         resp = await self.cell.getHealthCheck()
         return self.sendRestRetn(resp)
@@ -870,7 +900,7 @@ class StormVarsGetV1(Handler):
         varname = str(body.get('name'))
         defvalu = body.get('default')
 
-        if not await self.reqAuthAllowed(('globals', 'get', varname)):
+        if not await self.allowed(('globals', 'get', varname)):
             return
 
         valu = await self.cell.getStormVar(varname, default=defvalu)
@@ -887,7 +917,7 @@ class StormVarsPopV1(Handler):
         varname = str(body.get('name'))
         defvalu = body.get('default')
 
-        if not await self.reqAuthAllowed(('globals', 'pop', varname)):
+        if not await self.allowed(('globals', 'pop', varname)):
             return
 
         valu = await self.cell.popStormVar(varname, default=defvalu)
@@ -906,7 +936,7 @@ class StormVarsSetV1(Handler):
         if varvalu is s_common.novalu:
             return self.sendRestErr('BadArg', 'The "value" field is required.')
 
-        if not await self.reqAuthAllowed(('globals', 'set', varname)):
+        if not await self.allowed(('globals', 'set', varname)):
             return
 
         await self.cell.setStormVar(varname, varvalu)
@@ -939,3 +969,16 @@ class OnePassIssueV1(Handler):
         await self.cell.auth.setUserInfo(useriden, 'onepass', onepass)
 
         return self.sendRestRetn(passwd)
+
+class CoreInfoV1(Handler):
+    '''
+    /api/v1/core/info
+    '''
+
+    async def get(self):
+
+        if not await self.reqAuthUser():
+            return
+
+        resp = await self.cell.getCoreInfoV2()
+        return self.sendRestRetn(resp)
