@@ -1,23 +1,56 @@
 import logging
 
 import synapse.exc as s_exc
+import synapse.common as s_common
+
+import synapse.lib.layer as s_layer
 
 logger = logging.getLogger(__name__)
 
-maxvers = (0, 2, 0)
+maxvers = (0, 2, 1)
 
 class ModelRev:
 
     def __init__(self, core):
         self.core = core
         self.revs = (
-            # ((0, 0, 0), self._addModelVers),
-            # ((0, 1, 0), self._addFormNameSpaces),
-            # ((0, 1, 1), self._normContactAddress),
-            # ((0, 1, 2), self._afterUniqueIdens),
-            # ((0, 1, 3), self._ouOrgDissolve),
+            ((0, 2, 1), self.revModel20210126),
         )
 
+    async def revModel20210126(self, layers):
+
+        for layr in layers:
+
+            nodeedits = []
+            meta = {'time': s_common.now(), 'user': self.core.auth.rootuser.iden}
+
+            # uniq values of some array types....
+            def uniq(valu):
+                return tuple({v: True for v in valu}.keys())
+
+            async def save():
+                await layr.storNodeEdits(nodeedits, meta)
+                nodeedits.clear()
+
+            stortype = s_layer.STOR_TYPE_GUID | s_layer.STOR_FLAG_ARRAY
+            async for buid, propvalu in layr.iterPropRows('ou:org', 'industries'):
+
+                uniqvalu = uniq(propvalu)
+                if uniqvalu == propvalu:
+                    continue
+
+                nodeedits.append(
+                    (buid, 'ou:org', (
+                        (s_layer.EDIT_PROP_SET, ('industries', uniqvalu, propvalu, stortype), ()),
+                    )),
+                )
+
+                if len(nodeedits) >= 1000:
+                    await save()
+
+            if nodeedits:
+                await save()
+
     async def revCoreLayers(self):
 
         version = self.revs[-1][0] if self.revs else maxvers
