@@ -1115,8 +1115,11 @@ class Ival(Type):
     def _normPyIter(self, valu):
         (minv, maxv), info = self._normByTickTock(valu)
 
+        if minv == maxv:
+            maxv = maxv + 1
+
         # Norm via iter must produce an actual range.
-        if minv >= maxv:
+        if minv > maxv:
             raise s_exc.BadTypeValu(name=self.name, valu=valu,
                                     mesg='Ival range must in (min, max) format')
 
