@@ -41,7 +41,7 @@ class DeprecatedModel(s_module.CoreModule):
 
 class DataModelTest(s_t_utils.SynTest):
 
-    async def test_datmodel_formname(self):
+    async def test_datamodel_formname(self):
         modl = s_datamodel.Model()
         mods = (
             ('hehe', {
@@ -57,6 +57,24 @@ class DataModelTest(s_t_utils.SynTest):
         with self.raises(s_exc.BadFormDef):
             modl.addDataModels(mods)
 
+    async def test_datamodel_no_interface(self):
+        modl = s_datamodel.Model()
+        mods = (
+            ('hehe', {
+                'types': (
+                    ('test:derp', ('int', {}), {
+                        'interfaces': ('foo:bar',),
+                    }),
+                ),
+                'forms': (
+                    ('test:derp', {}, ()),
+                ),
+            }),
+        )
+
+        with self.raises(s_exc.NoSuchName):
+            modl.addDataModels(mods)
+
     async def test_datamodel_dynamics(self):
 
         modl = s_datamodel.Model()
@@ -88,8 +106,10 @@ class DataModelTest(s_t_utils.SynTest):
         with self.raises(s_exc.NoSuchUniv):
             modl.delUnivProp('newp')
 
+        modl.addIface('test:iface', {})
+
         modl.addType('bar', 'int', {}, {})
-        modl.addType('foo:foo', 'int', {}, {})
+        modl.addType('foo:foo', 'int', {}, {'interfaces': ('test:iface',)})
 
         modl.addForm('foo:foo', {}, ())
         modl.addFormProp('foo:foo', 'bar', ('bar', {}), {})
@@ -97,6 +117,8 @@ class DataModelTest(s_t_utils.SynTest):
         with self.raises(s_exc.CantDelType):
             modl.delType('bar')
 
+        modl.delForm('foo:foo')
+
     async def test_datamodel_del_prop(self):
 
         modl = s_datamodel.Model()
@@ -211,3 +233,15 @@ class DataModelTest(s_t_utils.SynTest):
             with self.getAsyncLoggerStream('synapse.cortex', mesg) as cstream:
                 async with await s_cortex.Cortex.anit(dirn, conf) as core:
                     self.true(await cstream.wait(6))
+
+    async def test_datamodel_getmodeldefs(self):
+        '''
+        Make sure you can make a new model with the output of datamodel.getModelDefs
+        '''
+        modl = s_datamodel.Model()
+        modl.addIface('test:iface', {})
+        modl.addType('foo:foo', 'int', {}, {'interfaces': ('test:iface',)})
+        modl.addForm('foo:foo', {}, ())
+        mdef = modl.getModelDefs()
+        modl2 = s_datamodel.Model()
+        modl2.addDataModels(mdef)
