@@ -1,6 +1,7 @@
 import os
 import sys
 import time
+import tarfile
 import asyncio
 
 from unittest import mock
@@ -27,6 +28,9 @@ def _exiterProc(pipe, srcdir, dstdir, lmdbpaths):
     pipe.send('captured')
     sys.exit(1)
 
+def _backupSleep(path, linkinfo, done):
+    time.sleep(3.0)
+
 class EchoAuthApi(s_cell.CellApi):
 
     def isadmin(self):
@@ -318,6 +322,10 @@ class CellTest(s_t_utils.SynTest):
                 self.eq('haha', await proxy.getUserProfInfo(visi.iden, 'hehe'))
                 self.eq('haha', (await proxy.getUserProfile(visi.iden))['hehe'])
 
+                iden = s_common.guid(('foo', 101))
+                udef = await proxy.addUser('foo', iden=iden)
+                self.eq(udef.get('iden'), iden)
+
     async def test_longpath(self):
         # This is similar to the DaemonTest::test_unixsock_longpath
         # but exercises the long-path failure inside of the cell's daemon
@@ -804,3 +812,129 @@ class CellTest(s_t_utils.SynTest):
                 await step()
 
                 self.none(await cell.delActiveCoro(s_common.guid()))
+
+    async def test_cell_stream_backup(self):
+
+        with self.getTestDir() as dirn:
+
+            backdirn = os.path.join(dirn, 'backups')
+            coredirn = os.path.join(dirn, 'cortex')
+            bkuppath = os.path.join(dirn, 'bkup.tar.gz')
+            bkuppath2 = os.path.join(dirn, 'bkup2.tar.gz')
+            bkuppath3 = os.path.join(dirn, 'bkup3.tar.gz')
+            bkuppath4 = os.path.join(dirn, 'bkup4.tar.gz')
+
+            conf = {'backup:dir': backdirn}
+            s_common.yamlsave(conf, coredirn, 'cell.yaml')
+
+            async with self.getTestCore(dirn=coredirn) as core:
+
+                nodes = await core.nodes('[test:str=streamed]')
+                self.len(1, nodes)
+
+                async with core.getLocalProxy() as proxy:
+
+                    with self.raises(s_exc.BadArg):
+                        async for msg in proxy.iterBackupArchive('nope'):
+                            pass
+
+                    await proxy.runBackup(name='bkup')
+
+                    with mock.patch('synapse.lib.cell._iterBackupProc', _backupSleep):
+                        arch = s_t_utils.alist(proxy.iterBackupArchive('bkup'))
+                        with self.raises(asyncio.TimeoutError):
+                            await asyncio.wait_for(arch, timeout=0.1)
+
+                        async def _fakeBackup(self, name=None, wait=True):
+                            s_common.gendir(os.path.join(backdirn, name))
+
+                        with mock.patch.object(s_cell.Cell, 'runBackup', _fakeBackup):
+                            arch = s_t_utils.alist(proxy.iterNewBackupArchive('nobkup', remove=True))
+                            with self.raises(asyncio.TimeoutError):
+                                await asyncio.wait_for(arch, timeout=0.1)
+
+                        async def _slowFakeBackup(self, name=None, wait=True):
+                            s_common.gendir(os.path.join(backdirn, name))
+                            await asyncio.sleep(3.0)
+
+                        with mock.patch.object(s_cell.Cell, 'runBackup', _slowFakeBackup):
+                            arch = s_t_utils.alist(proxy.iterNewBackupArchive('nobkup2', remove=True))
+                            with self.raises(asyncio.TimeoutError):
+                                await asyncio.wait_for(arch, timeout=0.1)
+
+                    with self.raises(s_exc.BadArg):
+                        async for msg in proxy.iterNewBackupArchive('bkup'):
+                            pass
+
+                    # Get an existing backup
+                    with open(bkuppath, 'wb') as bkup:
+                        async for msg in proxy.iterBackupArchive('bkup'):
+                            bkup.write(msg)
+
+                    # Create a new backup
+                    nodes = await core.nodes('[test:str=freshbkup]')
+                    self.len(1, nodes)
+
+                    with open(bkuppath2, 'wb') as bkup2:
+                        async for msg in proxy.iterNewBackupArchive('bkup2'):
+                            bkup2.write(msg)
+
+                    self.eq(('bkup', 'bkup2'), sorted(await proxy.getBackups()))
+                    self.true(os.path.isdir(os.path.join(backdirn, 'bkup2')))
+
+                    # Create a new backup and remove after
+                    nodes = await core.nodes('[test:str=lastbkup]')
+                    self.len(1, nodes)
+
+                    with open(bkuppath3, 'wb') as bkup3:
+                        async for msg in proxy.iterNewBackupArchive('bkup3', remove=True):
+                            bkup3.write(msg)
+
+                    self.eq(('bkup', 'bkup2'), sorted(await proxy.getBackups()))
+                    self.false(os.path.isdir(os.path.join(backdirn, 'bkup3')))
+
+                    # Create a new backup without a name param
+                    nodes = await core.nodes('[test:str=noname]')
+                    self.len(1, nodes)
+
+                    with open(bkuppath4, 'wb') as bkup4:
+                        async for msg in proxy.iterNewBackupArchive(remove=True):
+                            bkup4.write(msg)
+
+                    self.eq(('bkup', 'bkup2'), sorted(await proxy.getBackups()))
+
+            with tarfile.open(bkuppath, 'r:gz') as tar:
+                tar.extractall(path=dirn)
+
+            bkupdirn = os.path.join(dirn, 'bkup')
+            async with self.getTestCore(dirn=bkupdirn) as core:
+                nodes = await core.nodes('test:str=streamed')
+                self.len(1, nodes)
+
+                nodes = await core.nodes('test:str=freshbkup')
+                self.len(0, nodes)
+
+            with tarfile.open(bkuppath2, 'r:gz') as tar:
+                tar.extractall(path=dirn)
+
+            bkupdirn2 = os.path.join(dirn, 'bkup2')
+            async with self.getTestCore(dirn=bkupdirn2) as core:
+                nodes = await core.nodes('test:str=freshbkup')
+                self.len(1, nodes)
+
+            with tarfile.open(bkuppath3, 'r:gz') as tar:
+                tar.extractall(path=dirn)
+
+            bkupdirn3 = os.path.join(dirn, 'bkup3')
+            async with self.getTestCore(dirn=bkupdirn3) as core:
+                nodes = await core.nodes('test:str=lastbkup')
+                self.len(1, nodes)
+
+            with tarfile.open(bkuppath4, 'r:gz') as tar:
+                bkupname = os.path.commonprefix(tar.getnames())
+                tar.extractall(path=dirn)
+
+            bkupdirn4 = os.path.join(dirn, bkupname)
+            async with self.getTestCore(dirn=bkupdirn4) as core:
+                nodes = await core.nodes('test:str=noname')
+                self.len(1, nodes)
