@@ -53,3 +53,14 @@ class ModelRevTest(s_tests.SynTest):
 
                 self.true(layr.woot)
                 self.eq((9999, 9999, 9999), await layr.getModelVers())
+
+    async def test_modelrev_2_0_1(self):
+        async with self.getRegrCore('model-2.0.1') as core:
+
+            nodes = await core.nodes('ou:org=b084f448ee7f95a7e0bc1fd7d3d7fd3b')
+            self.len(1, nodes)
+            self.len(3, nodes[0].get('industries'))
+
+            nodes = await core.nodes('ou:org=57c2dd4feee21204b1a989b9a796a89d')
+            self.len(1, nodes)
+            self.len(1, nodes[0].get('industries'))
