@@ -191,6 +191,9 @@ class StormTest(s_t_utils.SynTest):
             with self.raises(s_exc.AuthDeny):
                 await core.callStorm(wget, opts=opts)
 
+            with self.raises(s_exc.AuthDeny):
+                await core.nodes('for $x in $lib.axon.list() { $lib.print($x) }', opts=opts)
+
             # test wget runtsafe / per-node / per-node with cmdopt
             nodes = await core.nodes(f'wget https://127.0.0.1:{port}/api/v1/active')
             self.len(1, nodes)
@@ -208,10 +211,40 @@ class StormTest(s_t_utils.SynTest):
             nodes = await core.nodes(f'wget https://127.0.0.1:{port}/api/v1/active | -> file:bytes +:name=active')
             self.len(1, nodes)
             self.eq(nodes[0].ndef[0], 'file:bytes')
+            sha256, size, created = nodes[0].get('sha256'), nodes[0].get('size'), nodes[0].get('.created')
+
+            items = await core.callStorm('$x=$lib.list() for $i in $lib.axon.list() { $x.append($i) } return($x)')
+            self.eq([(0, sha256, size)], items)
+
+            # test $lib.axon.del()
+            delopts = {'user': visi.iden, 'vars': {'sha256': sha256}}
+            with self.raises(s_exc.AuthDeny):
+                await core.callStorm('$lib.axon.del($sha256)', opts=delopts)
+            with self.raises(s_exc.AuthDeny):
+                await core.callStorm('$lib.axon.dels(($sha256,))', opts=delopts)
+            with self.raises(s_exc.BadArg):
+                await core.callStorm('$lib.axon.dels(newp)')
+            delopts = {'vars': {'sha256': sha256}}
+            self.eq((True, False), await core.callStorm('return($lib.axon.dels(($sha256, $sha256)))', opts=delopts))
+            self.false(await core.callStorm('return($lib.axon.del($sha256))', opts=delopts))
+
+            items = await core.callStorm('$x=$lib.list() for $i in $lib.axon.list() { $x.append($i) } return($x)')
+            self.len(0, items)
 
             msgs = await core.stormlist(f'wget https://127.0.0.1:{port}/api/v1/newp')
             self.stormIsInWarn('HTTP code 404', msgs)
 
+            self.len(1, await core.callStorm('$x=$lib.list() for $i in $lib.axon.list() { $x.append($i) } return($x)'))
+
+            size, sha256 = await core.callStorm('return($lib.bytes.put($buf))', opts={'vars': {'buf': b'foo'}})
+
+            items = await core.callStorm('$x=$lib.list() for $i in $lib.axon.list() { $x.append($i) } return($x)')
+            self.len(2, items)
+            self.eq((2, sha256, size), items[1])
+
+            items = await core.callStorm('$x=$lib.list() for $i in $lib.axon.list(2) { $x.append($i) } return($x)')
+            self.eq([(2, sha256, size)], items)
+
             # test request timeout
             async def timeout(self):
                 await asyncio.sleep(2)
@@ -388,6 +421,14 @@ class StormTest(s_t_utils.SynTest):
             self.stormIsInPrint('3496c02183961db4fbc179f0ceb5526347b37d8ff278279917b6eb6d39e1e272 inet:fqdn:iszone = True', msgs)
             self.stormIsInPrint('3496c02183961db4fbc179f0ceb5526347b37d8ff278279917b6eb6d39e1e272 inet:fqdn:zone = mvmnasde.com', msgs)
 
+            # test that a user without perms can diff but not apply
+            await visi.addRule((True, ('view', 'read')))
+            async with core.getLocalProxy(user='visi') as asvisi:
+                await self.agenraises(s_exc.AuthDeny, asvisi.eval('merge --diff --apply', opts={'view': view}))
+
+                msgs = await alist(asvisi.storm('ps:person | merge --diff', opts={'view': view}))
+                self.stormIsInPrint('inet:fqdn = mvmnasde.com', msgs)
+
             # merge all the nodes with anything stored in the top layer...
             await core.callStorm('''
                 for ($buid, $sode) in $lib.view.get().layers.0.getStorNodes() {
@@ -427,6 +468,37 @@ class StormTest(s_t_utils.SynTest):
             task = core.schedCoro(sleeper())
             self.false(await s_coro.waittask(task, timeout=0.1))
 
+            # test subquery based property assignment
+            await core.nodes('[(ou:industry=* :name=foo)] [(ou:industry=* :name=bar)] [+#sqa]')
+            nodes = await core.nodes('[ ou:org=* :alias=visiacme :industries={ou:industry#sqa}]')
+            self.len(1, nodes)
+            self.len(2, nodes[0].get('industries'))
+
+            nodes = await core.nodes('[ ps:contact=* :org={ou:org:alias=visiacme}]')
+            self.len(1, nodes)
+            self.nn(nodes[0].get('org'))
+
+            nodes = await core.nodes('ou:org:alias=visiacme [ :industries-={ou:industry:name=foo} ]')
+            self.len(1, nodes)
+            self.len(1, nodes[0].get('industries'))
+
+            nodes = await core.nodes('ou:org:alias=visiacme [ :industries+={ou:industry:name=foo} ]')
+            self.len(1, nodes)
+            self.len(2, nodes[0].get('industries'))
+
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('ou:org:alias=visiacme [ :name={} ]')
+
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('ou:org:alias=visiacme [ :name={[it:dev:str=hehe it:dev:str=haha]} ]')
+
+            with self.raises(s_exc.BadTypeValu):
+                await core.nodes('ou:org:alias=visiacme [ :industries={[inet:ipv4=1.2.3.0/24]} ]')
+
+            await core.nodes('ou:org:alias=visiacme [ :name?={} ]')
+            await core.nodes('ou:org:alias=visiacme [ :name?={[it:dev:str=hehe it:dev:str=haha]} ]')
+            await core.nodes('ou:org:alias=visiacme [ :industries?={[inet:ipv4=1.2.3.0/24]} ]')
+
     async def test_storm_pipe(self):
 
         async with self.getTestCore() as core:
@@ -729,6 +801,26 @@ class StormTest(s_t_utils.SynTest):
                                         ('woah.beep', 'note'): 'oh my',
                                         })
 
+            # Test perms
+            visi = await core.auth.addUser('visi')
+            await visi.setPasswd('secret')
+
+            async with core.getLocalProxy(user='visi') as asvisi:
+                with self.raises(s_exc.AuthDeny):
+                    await asvisi.callStorm(f'movetag woah perm')
+
+                await visi.addRule((True, ('node', 'tag', 'del', 'woah')))
+
+                with self.raises(s_exc.AuthDeny):
+                    await asvisi.callStorm(f'movetag woah perm')
+
+                await visi.addRule((True, ('node', 'tag', 'add', 'perm')))
+
+                await asvisi.callStorm(f'movetag woah perm')
+
+            self.len(0, await core.nodes('#woah'))
+            self.len(1, await core.nodes('#perm'))
+
         # Sad path
         async with self.getTestCore() as core:
             # Test moving a tag to itself
@@ -1661,6 +1753,31 @@ class StormTest(s_t_utils.SynTest):
             self.len(0, await core.nodes('test:str=refs <(refs)- *'))
             self.len(0, await core.nodes('test:str=* <(seen)- *'))
 
+            # Test perms
+            visi = await core.auth.addUser('visi')
+            await visi.setPasswd('secret')
+
+            await core.nodes('test:str=test1 [ +(refs)> { test:int=7 } ]')
+            self.len(1, await core.nodes('test:str=test1 -(refs)> *'))
+
+            async with core.getLocalProxy(user='visi') as asvisi:
+                with self.raises(s_exc.AuthDeny):
+                    await asvisi.callStorm('test:str=test1 | edges.del refs')
+
+                await visi.addRule((True, ('node', 'edge', 'del', 'refs')))
+
+                await asvisi.callStorm('test:str=test1 | edges.del refs')
+                self.len(0, await core.nodes('test:str=test1 -(refs)> *'))
+
+                await core.nodes('test:str=test1 [ +(refs)> { test:int=7 } ]')
+                with self.raises(s_exc.AuthDeny):
+                    await asvisi.callStorm('test:str=test1 | edges.del *')
+
+                await visi.addRule((True, ('node', 'edge', 'del')))
+
+                await asvisi.callStorm('test:str=test1 | edges.del *')
+                self.len(0, await core.nodes('test:str=test1 -(refs)> *'))
+
     async def test_storm_pushpull(self):
 
         with self.getTestDir() as dirn:
@@ -1713,6 +1830,12 @@ class StormTest(s_t_utils.SynTest):
                 purl = await core.callStorm('for ($iden, $pdef) in $lib.layer.get($layr0).get(pushs) { return($pdef.url) }', opts=opts)
                 self.true(purl.startswith('tcp://root:****@127.0.0.1'))
 
+                msgs = await core.stormlist('layer.push.list $layr0', opts=opts)
+                self.stormIsInPrint('tcp://root:****@127.0.0.1', msgs)
+
+                msgs = await core.stormlist('layer.pull.list $layr2', opts=opts)
+                self.stormIsInPrint('tcp://root:****@127.0.0.1', msgs)
+
                 self.eq(2, len(core.activecoros) - actv)
                 tasks = await core.callStorm('return($lib.ps.list())')
                 self.len(1, [t for t in tasks if t.get('name').startswith('layer pull:')])
@@ -1772,6 +1895,39 @@ class StormTest(s_t_utils.SynTest):
                 self.none(await core.callStorm('return($lib.layer.get($layr2).delPull($lib.guid()))', opts=opts))
                 self.none(await core.callStorm('return($lib.layer.get($layr0).delPush($lib.guid()))', opts=opts))
 
+                msgs = await core.stormlist('layer.push.list $layr0', opts=opts)
+                self.stormIsInPrint('No pushes configured', msgs)
+
+                msgs = await core.stormlist('layer.pull.list $layr2', opts=opts)
+                self.stormIsInPrint('No pulls configured', msgs)
+
+                # Test storm command add/del
+                q = f'layer.push.add $layr0 "tcp://root:secret@127.0.0.1:{port}/*/layer/{layr1}"'
+                msgs = await core.stormlist(q, opts=opts)
+                self.stormIsInPrint('Layer push added', msgs)
+
+                q = f'layer.pull.add $layr2 "tcp://root:secret@127.0.0.1:{port}/*/layer/{layr1}"'
+                msgs = await core.stormlist(q, opts=opts)
+                self.stormIsInPrint('Layer pull added', msgs)
+
+                msgs = await core.stormlist('layer.push.list $layr0', opts=opts)
+                self.stormIsInPrint('tcp://root:****@127.0.0.1', msgs)
+
+                msgs = await core.stormlist('layer.pull.list $layr2', opts=opts)
+                self.stormIsInPrint('tcp://root:****@127.0.0.1', msgs)
+
+                pidn = await core.callStorm('for ($iden, $pdef) in $lib.layer.get($layr0).get(pushs) { return($iden) }', opts=opts)
+                msgs = await core.stormlist(f'layer.push.del $layr0 {pidn}', opts=opts)
+                self.stormIsInPrint('Layer push deleted', msgs)
+                msgs = await core.stormlist('layer.push.list $layr0', opts=opts)
+                self.stormIsInPrint('No pushes configured', msgs)
+
+                pidn = await core.callStorm('for ($iden, $pdef) in $lib.layer.get($layr2).get(pulls) { return($iden) }', opts=opts)
+                msgs = await core.stormlist(f'layer.pull.del $layr2 {pidn}', opts=opts)
+                self.stormIsInPrint('Layer pull deleted', msgs)
+                msgs = await core.stormlist('layer.pull.list $layr2', opts=opts)
+                self.stormIsInPrint('No pulls configured', msgs)
+
                 # add a push/pull and remove the layer to cancel it...
                 await core.callStorm(f'$lib.layer.get($layr0).addPush("tcp://root:secret@127.0.0.1:{port}/*/layer/{layr1}")', opts=opts)
                 await core.callStorm(f'$lib.layer.get($layr2).addPull("tcp://root:secret@127.0.0.1:{port}/*/layer/{layr1}")', opts=opts)
@@ -1864,3 +2020,157 @@ class StormTest(s_t_utils.SynTest):
                 layr.logedits = False
                 with self.raises(s_exc.BadArg):
                     await layr.waitEditOffs(200)
+
+    async def test_storm_tagprune(self):
+
+        async with self.getTestCore() as core:
+
+            async with await core.snap() as snap:
+                node = await snap.addNode('test:str', 'foo')
+                await node.addTag('parent.child.grandchild')
+
+                node = await snap.addNode('test:str', 'bar')
+                await node.addTag('parent.childtag')
+                await node.addTag('parent.child.step')
+                await node.addTag('parent.child.grandchild')
+
+                node = await snap.addNode('test:str', 'baz')
+                await node.addTag('parent.child.step')
+                await node.addTag('parent.child.step.two')
+                await node.addTag('parent.child.step.three')
+
+            # Won't do anything but should work
+            nodes = await core.nodes('test:str | tag.prune')
+            self.len(3, nodes)
+
+            node = (await core.nodes('test:str=foo'))[0]
+            exp = [
+                'parent',
+                'parent.child',
+                'parent.child.grandchild'
+            ]
+            self.eq(list(node.tags.keys()), exp)
+
+            node = (await core.nodes('test:str=bar'))[0]
+            exp = [
+                'parent',
+                'parent.childtag',
+                'parent.child',
+                'parent.child.step',
+                'parent.child.grandchild'
+            ]
+            self.eq(list(node.tags.keys()), exp)
+
+            node = (await core.nodes('test:str=baz'))[0]
+            exp = [
+                'parent',
+                'parent.child',
+                'parent.child.step',
+                'parent.child.step.two',
+                'parent.child.step.three'
+            ]
+            self.eq(list(node.tags.keys()), exp)
+
+            await core.nodes('test:str | tag.prune parent.child.grandchild')
+
+            # Should remove all tags
+            node = (await core.nodes('test:str=foo'))[0]
+            self.eq(list(node.tags.keys()), [])
+
+            # Should only remove parent.child.grandchild
+            node = (await core.nodes('test:str=bar'))[0]
+            exp = ['parent', 'parent.childtag', 'parent.child', 'parent.child.step']
+            self.eq(list(node.tags.keys()), exp)
+
+            await core.nodes('test:str | tag.prune parent.child.step')
+
+            # Should only remove parent.child.step and parent.child
+            node = (await core.nodes('test:str=bar'))[0]
+            self.eq(list(node.tags.keys()), ['parent', 'parent.childtag'])
+
+            # Should remove all tags
+            node = (await core.nodes('test:str=baz'))[0]
+            self.eq(list(node.tags.keys()), [])
+
+            async with await core.snap() as snap:
+                node = await snap.addNode('test:str', 'foo')
+                await node.addTag('tag.tree.one')
+                await node.addTag('tag.tree.two')
+                await node.addTag('another.tag.tree')
+
+                node = await snap.addNode('test:str', 'baz')
+                await node.addTag('tag.tree.one')
+                await node.addTag('tag.tree.two')
+                await node.addTag('another.tag.tree')
+                await node.addTag('more.tags.to.remove')
+                await node.addTag('tag.that.stays')
+
+            # Remove multiple tags
+            tags = '''
+                tag.tree.one
+                tag.tree.two
+                another.tag.tree
+                more.tags.to.remove
+            '''
+            await core.nodes(f'test:str | tag.prune {tags}')
+
+            node = (await core.nodes('test:str=foo'))[0]
+            self.eq(list(node.tags.keys()), [])
+
+            node = (await core.nodes('test:str=baz'))[0]
+            exp = ['tag', 'tag.that', 'tag.that.stays']
+            self.eq(list(node.tags.keys()), exp)
+
+            async with await core.snap() as snap:
+                node = await snap.addNode('test:str', 'runtsafety')
+                await node.addTag('runtsafety')
+
+                node = await snap.addNode('test:str', 'foo')
+                await node.addTag('runtsafety')
+
+                node = await snap.addNode('test:str', 'runt.safety.two')
+                await node.addTag('runt.safety.two')
+                await node.addTag('runt.child')
+
+            # Test non-runtsafe usage
+            await core.nodes('test:str | tag.prune $node.value()')
+
+            node = (await core.nodes('test:str=runtsafety'))[0]
+            self.eq(list(node.tags.keys()), [])
+
+            node = (await core.nodes('test:str=foo'))[0]
+            self.eq(list(node.tags.keys()), ['runtsafety'])
+
+            node = (await core.nodes('test:str=runt.safety.two'))[0]
+            self.eq(list(node.tags.keys()), ['runt', 'runt.child'])
+
+            async with await core.snap() as snap:
+                node = await snap.addNode('test:str', 'foo')
+                await node.addTag('runt.need.perms')
+
+                node = await snap.addNode('test:str', 'runt.safety.two')
+                await node.addTag('runt.safety.two')
+
+            # Test perms
+            visi = await core.auth.addUser('visi')
+            await visi.setPasswd('secret')
+
+            async with core.getLocalProxy(user='visi') as asvisi:
+                with self.raises(s_exc.AuthDeny):
+                    await asvisi.callStorm(f'test:str | tag.prune runt.need.perms')
+
+                with self.raises(s_exc.AuthDeny):
+                    await asvisi.callStorm(f'test:str | tag.prune $node.value()')
+
+            await visi.addRule((True, ('node', 'tag', 'del', 'runt')))
+
+            async with core.getLocalProxy(user='visi') as asvisi:
+                await asvisi.callStorm(f'test:str | tag.prune runt.need.perms')
+
+                node = (await core.nodes('test:str=foo'))[0]
+                self.eq(list(node.tags.keys()), ['runtsafety'])
+
+                await asvisi.callStorm(f'test:str=runt.safety.two | tag.prune $node.value()')
+
+                node = (await core.nodes('test:str=runt.safety.two'))[0]
+                self.eq(list(node.tags.keys()), ['runt', 'runt.child'])
