@@ -461,6 +461,11 @@ class SynModelTest(s_t_utils.SynTest):
                 self.len(1, nodes)
                 self.eq(('syn:form', 'inet:fqdn'), nodes[0].ndef)
 
+                nodes = await core.nodes('syn:cmd=foobar :input -+> *')
+                self.len(3, nodes)
+                self.eq({('syn:form', 'inet:ipv4'), ('syn:form', 'inet:ipv6'), ('syn:cmd', 'foobar')},
+                        {n.ndef for n in nodes})
+
                 nodes = await core.nodes('syn:cmd +:input*[=inet:ipv4]')
                 self.len(1, nodes)
 
