@@ -1,6 +1,6 @@
 import hashlib
 
-import synapse.servers.axon as s_s_axon
+import synapse.axon as s_axon
 
 import synapse.tests.utils as s_t_utils
 
@@ -10,7 +10,7 @@ class AxonServerTest(s_t_utils.SynTest):
 
     async def test_server(self):
 
-        with self.getTestDir() as dirn:
+        with self.getTestDir() as dirn, self.withSetLoggingMock() as mock:
 
             outp = self.getTestOutp()
 
@@ -18,7 +18,8 @@ class AxonServerTest(s_t_utils.SynTest):
                     '--telepath', 'tcp://127.0.0.1:0/',
                     '--https', '0',
                     '--name', 'teleaxon']
-            async with await s_s_axon.main(argv, outp=outp) as axon:
+
+            async with await s_axon.Axon.initFromArgv(argv, outp=outp) as axon:
                 async with axon.getLocalProxy() as proxy:
                     async with await proxy.upload() as fd:
                         await fd.write(b'asdfasdf')
@@ -27,6 +28,6 @@ class AxonServerTest(s_t_utils.SynTest):
                 self.true(axon.dmon.shared.get('teleaxon') is axon)
 
             # And data persists...
-            async with await s_s_axon.main(argv, outp=outp) as axon:
+            async with await s_axon.Axon.initFromArgv(argv, outp=outp) as axon:
                 async with axon.getLocalProxy() as proxy:
                     self.true(await proxy.has(asdfhash))
