@@ -49,6 +49,7 @@ import synapse.daemon as s_daemon
 import synapse.cryotank as s_cryotank
 import synapse.telepath as s_telepath
 
+import synapse.lib.aha as s_aha
 import synapse.lib.coro as s_coro
 import synapse.lib.cmdr as s_cmdr
 import synapse.lib.hive as s_hive
@@ -83,7 +84,23 @@ def norm(z):
     return z
 
 class LibTst(s_stormtypes.Lib):
+    '''
+    LibTst for testing!
+    '''
+    _storm_locals = (
+        {'name': 'beep',
+         'desc': '''
+        Example storm func.
 
+        Notes:
+            It beeps strings!''',
+         'type': {'type': 'function', '_funcname': 'beep',
+                  'args': (
+                      {'name': 'valu', 'type': 'str', 'desc': 'The value to beep.', },
+                  ),
+                  'returns': {'type': 'str', 'desc': 'The beeped string.', }}},
+    )
+    _storm_lib_path = ('test',)
     def addLibFuncs(self):
         self.locls.update({
             'beep': self.beep,
@@ -1101,6 +1118,16 @@ class SynTest(unittest.TestCase):
 
                 yield core, prox, testsvc
 
+    @contextlib.asynccontextmanager
+    async def getTestAha(self, conf=None, dirn=None):
+        if dirn:
+            async with await s_aha.AhaCell.anit(dirn, conf=conf) as aha:
+                yield aha
+        else:
+            with self.getTestDir() as dirn:
+                async with await s_aha.AhaCell.anit(dirn, conf=conf) as aha:
+                    yield aha
+
     async def addSvcToCore(self, svc, core, svcname='svc'):
         '''
         Add a service to a Cortex using telepath over tcp.
@@ -1390,9 +1417,12 @@ class SynTest(unittest.TestCase):
     async def execToolMain(self, func, argv):
         outp = self.getTestOutp()
 
-        def execmain():
-            return func(argv, outp=outp)
-        retn = await s_coro.executor(execmain)
+        if inspect.iscoroutinefunction(func):
+            retn = await func(argv, outp=outp)
+        else:
+            def execmain():
+                return func(argv, outp=outp)
+            retn = await s_coro.executor(execmain)
         return retn, outp
 
     @contextlib.contextmanager
@@ -1659,7 +1689,7 @@ class SynTest(unittest.TestCase):
 
         Args:
             obj (object): Object being inspected. This is validated to be a
-            tuple of length two, contiaing a str or None as the first value,
+            tuple of length two, containing a str or None as the first value,
             and a dict as the second value.
 
         Notes:
